"""Evaluation CLI command."""

from argparse import Namespace


def run_evaluation(args: Namespace):
    """Run evaluation command."""
    print("Evaluation command coming soon.")
    # TODO: Implement evaluation

